'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

exports.default = function (babel) {
  var t = babel.types;

  var identifiers = new _set2.default();
  var declarators = new _set2.default();
  return {
    name: 'babel-plugin-glamorous-displayName',
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        var defaultSpecifierPath = path.get('specifiers')[0];
        if (path.node.source.value !== 'glamorous' || !t.isImportDefaultSpecifier(defaultSpecifierPath)) {
          return;
        }
        var name = defaultSpecifierPath.node.local.name;

        var _path$scope$getBindin = path.scope.getBinding(name),
            referencePaths = _path$scope$getBindin.referencePaths;

        referencePaths.forEach(function (reference) {
          identifiers.add(reference);
        });
      },
      VariableDeclarator: function VariableDeclarator(path) {
        var node = path.node;

        if (!isRequireCall(node.init) || !t.isIdentifier(node.id)) {
          return;
        }
        var name = node.id.name;

        var binding = path.scope.getBinding(name);
        if (binding) {
          var referencePaths = binding.referencePaths;

          referencePaths.forEach(function (reference) {
            identifiers.add(reference);
          });
        }
      },

      Program: {
        exit: function exit(path, _ref) {
          var filename = _ref.file.opts.filename;

          (0, _from2.default)(identifiers).forEach(function (identifier) {
            var declarator = identifier.findParent(t.isVariableDeclarator);
            if (!declarator || declarators.has(declarator) || !t.isCallExpression(declarator.node.init)) {
              return;
            }
            declarators.add(declarator);
            var _declarator = declarator,
                name = _declarator.node.id.name;

            var displayName = name;
            if (filename && filename !== 'unknown') {
              displayName = nodePath.parse(filename).name + '__' + displayName;
            }

            if (declarator.parentPath.findParent(t.isExportNamedDeclaration)) {
              declarator = declarator.parentPath;
            }
            var lastBodyItem = getLastBodyItem(declarator.scope.path);
            var insertMethod = lastBodyItem.type === 'ReturnStatement' ? 'insertBefore' : 'insertAfter';

            lastBodyItem[insertMethod](t.expressionStatement(t.assignmentExpression('=', t.memberExpression(t.identifier(name), t.identifier('displayName')), t.stringLiteral(displayName))));
          });
        }
      }
    }
  };

  function isRequireCall(callExpression) {
    return callExpression && callExpression.type === 'CallExpression' && callExpression.callee.name === 'require' && callExpression.arguments.length === 1 && callExpression.arguments[0].value === 'glamorous';
  }
};

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var nodePath = require('path');

function getLastBodyItem(path) {
  var bodyItems = getBody(path);
  var last = bodyItems[bodyItems.length - 1];
  if (last.type === 'TryStatement') {
    if (last.node.finalizer) {
      last = getLastBodyItem(last.get('finalizer'));
    } else {
      last = getLastBodyItem(last.get('block'));
    }
  }
  return last;
}

function getBody(path) {
  if (path.node && path.node.body) {
    return getBody(path.get('body'));
  } else {
    return path;
  }
}